// DE_method.cpp: implementation of the CDE_method class.
//
//////////////////////////////////////////////////////////////////////

#include "DE_method.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CDE_method::CDE_method()
{
	max_iterations = Max_of_NFEs/population_size;
	max_NFFEs      = Max_of_NFEs;
	pop_size       = population_size;

	parent_pop   = new CIndividual[2*pop_size];
	child_pop    = new CIndividual[2*pop_size];
	child_pop1   = new CIndividual[2*pop_size];
	upper_bounds = new double[N_of_x];
	lower_bounds = new double[N_of_x];
	sort_index   = new int[2*pop_size];

	/************************************************************************/
	/* default parameter settings for DE method                             */
	/************************************************************************/
	m_F              = 0.5;
	m_CR             = 0.9;
	m_strategy       = 1;		// DE/rand/1/bin
	param_adaptation = 1;		// for parameter adaptation 

	/************************************************************************/
	/* for rank-based DE methods                                            */
	/************************************************************************/
	weights_flag = 2;			// 1: ranking-based linear model; 2: ranking-based quadratic model; 3: ranking-based cosine model

	/************************************************************************/
	/* for JADE methods                                                     */
	/************************************************************************/
	JADE_mu_CR = 0.5;
	JADE_mu_FF = 0.5;
}

CDE_method::~CDE_method()
{
	delete []parent_pop;
	delete []child_pop;
	delete []child_pop1;
	delete []upper_bounds;
	delete []lower_bounds;
	delete []sort_index;
}

void CDE_method::init_variables()
{
	if ((index_of_normal) == 0)
	{
		printf("Are you kinding me?\nThere is no function to be optimized.\n");
		exit(0);
	}

	if (index_of_normal != 0)
	{
		init_normal_variables();
	}

	max_interval = 0;
	double temp;
	for (int i=0;i<N_of_x;i++)
	{
		temp = upper_bounds[i]-lower_bounds[i];
		max_interval += temp*temp;
	}
	max_interval = sqrt(max_interval);
}

void CDE_method::init_normal_variables()
{
	int n     = N_of_x;
	func_flag = index_of_normal;

	switch(func_flag) {
	case 1:
		lower_bounds[0] = 0.0;	
		upper_bounds[0] = 1.2;		// E0
		lower_bounds[1] = 0.0;	
		upper_bounds[1] = 1.0;		// A
		lower_bounds[2] = 0.0;	
		upper_bounds[2] = 10.0;		// in
		lower_bounds[3] = 0.0;	
		upper_bounds[3] = 10.0;		// i0, may be extended
		lower_bounds[4] = 0.0;		
		upper_bounds[4] = 1.0;		// r
		lower_bounds[5] = 0.0;	
		upper_bounds[5] = 1.0;		// m
		lower_bounds[6] = 0.0;	
		upper_bounds[6] = 1.0;		// n
		
		known_optimal   = 0.0;
		MINIMIZE        = 1;
		break;
	case 2:
		lower_bounds[0] = 0.0;	
		upper_bounds[0] = 1.2;		// E0 <================
		lower_bounds[1] = 0.0;	
		upper_bounds[1] = 1.0;		// A
		lower_bounds[2] = 0.0;	
		upper_bounds[2] = 600.0;		// i0, may be extended <================
		lower_bounds[3] = 0.0;		
		upper_bounds[3] = 1.0;		// r
		lower_bounds[4] = 0.0;	
		upper_bounds[4] = 1.0;		// m
		lower_bounds[5] = 0.0;	
		upper_bounds[5] = 1.0;		// n
		
		known_optimal   = 0.0;
		MINIMIZE        = 1;
		break;
	case 3:
		lower_bounds[0] = 0.0;	
		upper_bounds[0] = 1.2;		// E0 <================
		lower_bounds[1] = 0.0;	
		upper_bounds[1] = 1.0;		// A
		lower_bounds[2] = 0.0;	
		upper_bounds[2] = 600.0;	// i0 <================
		lower_bounds[3] = 0.0;	
		upper_bounds[3] = 1.0;		// r
		lower_bounds[4] = 0.0;		
		upper_bounds[4] = 1.0;		// B
		lower_bounds[5] = 0.0;	
		upper_bounds[5] = 6000.0;	// iL <================
		
		known_optimal   = 0.0;
		MINIMIZE        = 1;
		break;
	case 4:
		lower_bounds[0] = 0.0;	
		upper_bounds[0] = 1.0;			// 
		lower_bounds[1] = 0.0;	
		upper_bounds[1] = 1.0;			// 
		lower_bounds[2] = -6000.0;	
		upper_bounds[2] = 0.0;			// 
		lower_bounds[3] = 500.0;	
		upper_bounds[3] = 1000.0;		// 
		lower_bounds[4] = 0.0;		
		upper_bounds[4] = 1.0;			// 
		lower_bounds[5] = 0.0;	
		upper_bounds[5] = 200.0;		// 
		lower_bounds[6] = 0.0;	
		upper_bounds[6] = 15000.0;		// 
		lower_bounds[7] = 0.0;		
		upper_bounds[7] = 1.0;			//
		lower_bounds[8] = 0.0;		
		upper_bounds[8] = 5000.0;		//
		lower_bounds[9] = 0.0;		
		upper_bounds[9] = 1.0;		//
		
		known_optimal   = 0.0;
		MINIMIZE        = 1;
		break;
	case 5:
		lower_bounds[0] = 0.0;	
		upper_bounds[0] = 1.2;		// E0 <================
		lower_bounds[1] = 0.0;	
		upper_bounds[1] = 1.0;		// A
		lower_bounds[2] = 0.0;	
		upper_bounds[2] = 30.0;		// i0		30 for A, 600 for mA
		lower_bounds[3] = 0.0;	
		upper_bounds[3] = 1.0;		// r
		lower_bounds[4] = 0.0;		
		upper_bounds[4] = 1.0;		// B
		lower_bounds[5] = 0.0;	
		upper_bounds[5] = 200.0;	// iL		200 for A, 6000 for mA
		lower_bounds[6] = 0.0;	
		upper_bounds[6] = 30.0;		// in		30 for A, 600 for mA
		
		known_optimal   = 0.0;
		MINIMIZE        = 1;
		break;
	default:
		printf("The function you selected does not exist in init_normal_variables().\n");
		exit(0);
	}
}

void CDE_method::init_pop()
{
	printf("Random based parent_pop initialization method is used.\n");
	init_pop_random();

	// initialize the F and CR of DE
	for (int i=0;i<pop_size;i++)
	{
		parent_pop[i].F  = m_rnd.rndreal(0.1, 1.0);
		parent_pop[i].CR = m_rnd.rndreal(0.0, 1.0);
	}
}

void CDE_method::init_pop_random()
{
	int i, j;
	for (i=0;i<pop_size;i++)
	{
		for (j=0;j<N_of_x;j++)
		{
			parent_pop[i].xreal[j] = m_rnd.rndreal(lower_bounds[j],upper_bounds[j]);
		}
	}
//	parent_pop[0].xreal[0] = 1.0313;
//	parent_pop[0].xreal[1] = 3.1756e-3;
//	parent_pop[0].xreal[2] = 1.2053;
//	parent_pop[0].xreal[3] = 1.0/0.0014;
//	parent_pop[0].xreal[4] = 48.2889;
}

void CDE_method::evaluate_ind(CIndividual &indv)
{
	if ((index_of_normal) == 0)
	{
		printf("Are you kinding me?\nThere is no function to be optimized.\n");
		exit(0);
	}

	if (index_of_normal != 0)
	{
		evaluate_normal_ind(indv);
	}

	// convert objective function value into fitness
	indv.fitness = MINIMIZE*indv.obj;

	// sum violation of the constrained functions
	if (N_of_constr == 0)
	{
		indv.constr_violation = 0.0;
		indv.feasible = 1;
	}
}

void CDE_method::evaluate_normal_ind(CIndividual &indv)
{
	func_flag = index_of_normal;

	m_func.evaluate_normal_fitness(indv.xreal,indv.obj,indv.constr,
		func_flag, evaluations);
}

void CDE_method::evaluate_pop(CIndividual *pop, int size)
{
	for (int i=0;i<size;i++)
	{
		evaluate_ind(pop[i]);
	}
}

int CDE_method::compare_ind (CIndividual *indv1, CIndividual *indv2)
{
	if((indv1->feasible==TRUE && indv2->feasible==TRUE))
	{
		if(indv1->fitness < indv2->fitness)
		{
			return 1;
		}
		else if (indv1->fitness > indv2->fitness)
		{
			return -1;
		}
		else
		{
			return 0;
		}
	}
	else if(indv1->feasible==TRUE && indv2->feasible==FALSE)
	{
		return 1;
	}
	else if (indv1->feasible==FALSE && indv2->feasible==TRUE)
	{
		return -1;
	}
	else
	{
		if(indv1->constr_violation < indv2->constr_violation)
		{
			return 1;
		}
		else if (indv1->constr_violation > indv2->constr_violation)
		{
			return -1;
		}
		else
		{
			//return 0;
			if (indv1->fitness < indv2->fitness)
			{
				return 1;
			}
			else if (indv1->fitness > indv2->fitness)
			{
				return -1;
			}
			else
			{
				return 0;
			}
		}
	}
}

void CDE_method::find_best_index(CIndividual *pop, int size)
{
	int flag;

	best_index = 0;
	for (int i=1;i<size;i++)
	{
		flag = compare_ind(&pop[i], &pop[best_index]);
		if (flag == 1)
		{
			best_index = i;
		}
	}

	worst_index = 0;
	for (i=1;i<size;i++)
	{
		flag = compare_ind(&pop[i], &pop[worst_index]);
		if (flag == -1)
		{
			worst_index = i;
		}
	}
}

void CDE_method::random_index(int *array_index, int all_size, int size)
{
	int i,j,krand;
	int *a;
	a = new int[all_size];

	for(i = 0;i<all_size;i++)
	{
		a[i] = i;
	}
	for(i=0;i<size;i++)
	{
		j     = m_rnd.rndint(i,(all_size-1));
		krand = a[i];
		a[i]  = a[j];
		a[j]  = krand;
	}	
	for(i=0;i<size;i++)
	{
		array_index[i] = a[i];
	}

	//a = NULL;
	delete []a;
}

void CDE_method::shell_sort_pop(CIndividual *pop, int *list, int size)
{
	int done;
	int step, bound, i, j;
	int temp;

	for (i=0;i<size;i++)
	{
		list[i] = i;
	}

	step = size;  // array length
	while (step > 1) 
	{
		step /= 2;	//halve the step size
		do 
		{
			done   = 1;
			bound  = size - step;
			for (j = 0; j < bound; j++) 
			{
				i = j + step + 1;
				if (compare_ind(&pop[list[j]], &pop[list[i-1]]) == -1) 	
				{
					temp      = list[i-1];
					list[i-1] = list[j];
					list[j]   = temp;
					done      = 0; // if a swap has been made we are not finished yet
				}  // if
			}  // for
		} while (done == 0);   // while
	} //while (step > 1)
}

void CDE_method::shell_sort_array(double *array, int *list, int size)
{
	int done;
	int step, bound, i, j;
	int temp;

	for (i=0;i<size;i++)
	{
		list[i] = i;
	}

	step = size;  // array length
	while (step > 1) 
	{
		step /= 2;	//halve the step size
		do 
		{
			done   = 1;
			bound  = size - step;
			for (j = 0; j < bound; j++) 
			{
				i = j + step + 1;
				if (array[list[j]] < array[list[i-1]]) 	
				{
					temp      = list[i-1];
					list[i-1] = list[j];
					list[j]   = temp;
					done      = 0; // if a swap has been made we are not finished yet
				}  // if
			}  // for
		} while (done == 0);   // while
	} //while (step > 1)
}

void CDE_method::shell_sort_array_t(tFitness *array, int *list, int size)
{
	int done;
	int step, bound, i, j;
	int temp;

	for (i=0;i<size;i++)
	{
		list[i] = i;
	}

	step = size;  // array length
	while (step > 1) 
	{
		step /= 2;	//halve the step size
		do 
		{
			done   = 1;
			bound  = size - step;
			for (j = 0; j < bound; j++) 
			{
				i = j + step + 1;
				if (array[list[j]] < array[list[i-1]]) 	
				{
					temp      = list[i-1];
					list[i-1] = list[j];
					list[j]   = temp;
					done      = 0; // if a swap has been made we are not finished yet
				}  // if
			}  // for
		} while (done == 0);   // while
	} //while (step > 1)
}

double CDE_method::median(double *array, int size)
{
	double value = 0.0;

	int list[2000];
	for (int i=0;i<2000;i++)
	{
		list[i] = i;
	}
	shell_sort_array(array, list, size);
	
	if (size%2 == 1)
	{// the size of the array is odd.
		int media_index;
		media_index = size/2;
		value = array[list[media_index]];
	}
	else
	{// the size of the array is even.
		int a = (size-1)/2;
		int b = size/2;
		value = (array[list[a]]+array[list[b]])/2.0;
	}

	return value;
}

double CDE_method::mean_std(double *array, int size, double &stdv)
{
	double value = 0.0;

	assert(size > 0);

	for (int i=0;i<size;i++)
	{
		value += array[i];
	}
	value = value/((double)size);

	stdv = 0.0;
	for (i=0;i<size;i++)
	{
		stdv += (array[i]-value)*(array[i]-value);
	}
	stdv = sqrt(stdv/((double)size-1));

	return value;
}

tFitness CDE_method::mean_std_t(tFitness *array, int size, tFitness &stdv)
{
	tFitness value = 0.0;

	assert(size > 0);

	for (int i=0;i<size;i++)
	{
		value += array[i];
	}
	value = value/((double)size);

	stdv = 0.0;
	for (i=0;i<size;i++)
	{
		stdv += (array[i]-value)*(array[i]-value);
	}
	stdv = sqrt(stdv/((double)size-1));

	return value;
}

void CDE_method::display_result(long int gen)
{
	if(gen%10==0 || gen == 1)
	{
		//ǰøĻ
		cout<<setw(5)<<gen;
		cout<<setw(8)<<evaluations;
		//ʾԼֵ
		if (N_of_constr != 0)
		{			
			cout<<setw(15)<<best_individual.constr_violation;
		}
		cout.precision(10);					//
		cout<<setw(20)<<MINIMIZE * parent_pop[best_index].fitness;
		cout<<setw(20)<<MINIMIZE * parent_pop[m_rnd.rndint(0, pop_size-1)].fitness;
		cout<<setw(20)<<MINIMIZE * parent_pop[worst_index].fitness<<endl;
	}
}

void CDE_method::report_result(long int gen, ofstream &file)
{
	if (gen < 5 )
	{
		//ǰøļ
		file<<setw(5)<<gen;
		file<<setw(10)<<evaluations;
		file.precision(20);			//
		file<<setw(30)<<MINIMIZE * best_individual.fitness;
		file<<setw(30)<<best_individual.constr_violation;
		//file<<setw(30)<<best_individual.CR;
		file<<endl;
	}
	else
	{
		if ((gen % (max_iterations/out_internal) == 0 ||
			gen >= max_iterations ||
			evaluations >= max_NFFEs) )
		{
			//ǰøļ
			file<<setw(5)<<gen;
			file<<setw(10)<<evaluations;
			file.precision(20);			//
			file<<setw(30)<<MINIMIZE * best_individual.fitness;
			file<<setw(30)<<best_individual.constr_violation;
			//file<<setw(30)<<best_individual.CR;
			file<<endl;
		}
	}
}

void CDE_method::report_diversity(long int gen, ofstream &file)
{
	// calculate the diversity of the population
	double x_average[2000];
	for (int i=0;i<2000;i++)
	{
		x_average[i] = 0.0;
	}

	for (i=0;i<N_of_x;i++)
	{
		for (int j=0;j<pop_size;j++)
		{
			x_average[i] += parent_pop[j].xreal[i];
		}
		x_average[i] = x_average[i]/((double)pop_size);
	}

	double diversity = 0.0;
	for (i=0;i<pop_size;i++)
	{
		double temp = 0.0;
		double x;
		for (int j=0;j<N_of_x;j++)
		{
			x = parent_pop[i].xreal[j];
			temp += (x-x_average[j])*(x-x_average[j]);
		}
		diversity += sqrt(temp);
	}
	diversity = diversity/(((double)pop_size)*max_interval);

	pop_diversity = diversity;
	
	if (gen < 100 )
	{
		//ǰøļ
		file<<setw(5)<<gen;
		file<<setw(10)<<evaluations;
		file.precision(20);			//
		file<<setw(30)<<diversity<<endl;
	}
	else
	{
		if ((gen % out_internal == 0 ||
			gen >= max_iterations ||
			evaluations >= max_NFFEs) )
		{
			//ǰøļ
			file<<setw(5)<<gen;
			file<<setw(10)<<evaluations;
			file.precision(20);			//
			file<<setw(30)<<diversity<<endl;
		}
	}
}

void CDE_method::report_parameter(long int gen, ofstream &file)
{	
	int counter = max_iterations/20;
	if (method_flag==5 || method_flag==6 )
	{
		if (gen <=10 || gen%counter == 0)
		{
			//ǰøļ
			file<<setw(5)<<gen;
			file<<setw(10)<<evaluations;
			file.precision(20);			//
			file<<setw(30)<<JADE_mu_CR;
			file<<setw(30)<<JADE_mu_FF;
			file<<setw(30)<<"0";
			file<<endl;
		}
	}
	else
	{
		if (gen <=10 || gen%counter == 0)
		{
			//ǰøļ
			file<<setw(5)<<gen;
			file<<setw(10)<<evaluations;
			file.precision(20);			//
			file<<setw(30)<<best_individual.CR;
			file<<setw(30)<<best_individual.F;
			file<<setw(30)<<"0";
			file<<endl;
		}
	}
}

void CDE_method::sort_pop(CIndividual *pop, int size)
{
	for (int i=0;i<size;i++)
	{
		sort_index[i] = i;
	}
	shell_sort_pop(pop, sort_index, size);
}

// copy files from the source directory to the target directory
void CDE_method::copy_files(char *source, char *target)
{
	FILE *fs,*ft;
	char ma[204];
	unsigned int rd=0;

	fs=fopen(source, "rb");
	ft=fopen(target, "wb"); 
	if (fs&&ft) 
	{
		while (!feof(fs))
		{
			rd=fread(ma,sizeof(char),204,fs);
			fwrite(ma,sizeof(char),rd,ft);
		}
		fclose(fs);
		fclose(ft);
	}
}

long CDE_method::filesize(FILE *stream)
{
	long curpos, length;
	curpos = ftell(stream);
	fseek(stream, 0L, SEEK_END);
	length = ftell(stream);
	fseek(stream, curpos, SEEK_SET);
	return length;
}

int CDE_method::copyfile(const char* src,const char* dest)
{
	FILE *fp1,*fp2;
	int fsize,factread;
	static unsigned char buffer[SIZEOFBUFFER];

	fp1=fopen(src,  "rb");
	fp2=fopen(dest, "wb+");
	if (!fp1 || !fp2) 
	{
		return 0;
	}

    for (fsize=filesize(fp1);fsize>0;fsize-=SIZEOFBUFFER)
    {
		factread=fread(buffer,1,SIZEOFBUFFER,fp1);
		fwrite(buffer,factread,1,fp2);
    }
	fclose(fp1);
	fclose(fp2);
	return 1;
}

// create folders, only for batch execution
void CDE_method::create_folders(int method_ii, int strategy_ii)
{
	int status;
	int func_index;
	if (index_of_normal)	func_index = index_of_normal;

	/************************************************************************/
	/* Create the zero-order folders: function level                         */
	/************************************************************************/
	char f_name_0[150];
	// set the folder name
	sprintf(f_name_0,"%s%s%d", ".\\results", (func_index<10 ? "\\f0" : "\\f"), func_index);

	// create the folders
	status = mkdir(f_name_0);
	//if (!status==0)	printf("Unable to create directory 1\n");
	/************************************************************************/

	/************************************************************************/
	/* Create the first-order folders: method level                         */
	/************************************************************************/
	char f_name0[150];
	// set the folder name
	sprintf(f_name0,"%s%s%d%s%d", ".\\results", (func_index<10 ? "\\f0" : "\\f"), func_index, "\\algorithm_", method_ii);

	// create the folders
	status = mkdir(f_name0);
	//if (!status==0)	printf("Unable to create directory 1\n");
	/************************************************************************/

	/************************************************************************/
	/* Create the second-order folders: strategy level                      */
	/************************************************************************/
	char f_name00[150];
	// set the folder name
	sprintf(f_name00,"%s%s%d%s%d%s%d", ".\\results", (func_index<10 ? "\\f0" : "\\f"), func_index, "\\algorithm_", method_ii, "\\strategy_", strategy_ii);

	// create the folders
	status = mkdir(f_name00);
	//if (!status==0)	printf("Unable to create directory 1\n");
	/************************************************************************/

	/************************************************************************/
	/* Copy the statistic file to the strategy_level folder                 */
	/************************************************************************/
	char source_file_name[150] = ".\\Results\\1-DA-50.exe";
	char target_file_name[150];
	sprintf(target_file_name, "%s%s%d%s%d%s%d%s", ".\\results", (func_index<10 ? "\\f0" : "\\f"), func_index, "\\algorithm_", method_ii, "\\strategy_", strategy_ii, "\\1-DA-50.exe");
	//copy_files(source_file_name, target_file_name);
	copyfile(source_file_name, target_file_name);

	/************************************************************************/
	/* Create the third-order folders: results level                        */
	/************************************************************************/
	char f_name1[150];
	char f_name2[150];
	char f_name3[150];
	// set the folder name
	sprintf(f_name1, "%s%s%d%s%d%s%d%s", ".\\results", (func_index<10 ? "\\f0" : "\\f"), func_index, "\\algorithm_", method_ii, "\\strategy_", strategy_ii, "\\process");
	sprintf(f_name2, "%s%s%d%s%d%s%d%s", ".\\results", (func_index<10 ? "\\f0" : "\\f"), func_index, "\\algorithm_", method_ii, "\\strategy_", strategy_ii, "\\diversity");
	sprintf(f_name3, "%s%s%d%s%d%s%d%s", ".\\results", (func_index<10 ? "\\f0" : "\\f"), func_index, "\\algorithm_", method_ii, "\\strategy_", strategy_ii, "\\migration");

	// create the folders
	status = mkdir(f_name1);
	//if (!status==0)	printf("Unable to create directory 2\n");
	status = mkdir(f_name2);
	//if (!status==0)	printf("Unable to create directory 3\n");
	status = mkdir(f_name3);
	//if (!status==0)	printf("Unable to create directory 4\n");
	/************************************************************************/
}

void CDE_method::Run_Optimizer(int run_no, double seed, int method, int strategy_ii)
{
	m_strategy = strategy_ii;

	if (batch_execution==1)
	{
		create_folders(method, strategy_ii);
	}
	
	printf("Population size: %d\n", pop_size);
	switch(method) {
	case 1:
		printf("Algorithm: rank_JADE_method.\n");
		break;
	default:
		printf("The method does not exist!\n");
		exit(0);
	}

	int i = 0; 
	
	method_flag = method;
	rnd_seed    = seed;

	int func_index;
	if (index_of_normal)	func_index = index_of_normal;

	ofstream SummaryFile;
	char     f_name0[150];
	if (batch_execution==0)	
	{
		SummaryFile.open(".\\results\\summary.txt",ios::app);
	}
	if (batch_execution==1) 
	{
		sprintf(f_name0, "%s%s%d%s%d%s%d%s", ".\\results", (func_index<10 ? ".\\f0" : ".\\f"), func_index, ".\\algorithm_", method, ".\\strategy_", strategy_ii, ".\\summary.txt");
		SummaryFile.open(f_name0, ios::app);
	}

	char f_name1[150];
	if (batch_execution==0)	
	{
		sprintf(f_name1,"%s%d%s",".\\results\\process\\process_",run_no+1,".txt");	
	}
	if (batch_execution==1) 
	{
		sprintf(f_name1, "%s%s%d%s%d%s%d%s%d%s", ".\\results", (func_index<10 ? ".\\f0" : ".\\f"), func_index, ".\\algorithm_", method, ".\\strategy_", strategy_ii, ".\\process/process_" ,run_no+1,".txt");
	}
	ofstream file_process(f_name1);

	char f_name2[150];
	if (batch_execution==0)	
	{
		sprintf(f_name2,"%s%d%s",".\\results\\diversity\\diversity_",run_no+1,".txt");	
	}
	if (batch_execution==1) 
	{
		sprintf(f_name2, "%s%s%d%s%d%s%d%s%d%s", ".\\results", (func_index<10 ? ".\\f0" : ".\\f"), func_index, ".\\algorithm_", method, ".\\strategy_", strategy_ii, ".\\diversity/diversity_",run_no+1,".txt");
	}
	ofstream file_diversity(f_name2);

	char f_name3[150];
	if (batch_execution==0)	
	{
		sprintf(f_name3,"%s%d%s",".\\results\\migration\\migration_",run_no+1,".txt");	
	}
	if (batch_execution==1) 
	{
		sprintf(f_name3, "%s%s%d%s%d%s%d%s%d%s", ".\\results", (func_index<10 ? ".\\f0" : ".\\f"), func_index, ".\\algorithm_", method, ".\\strategy_", strategy_ii, ".\\migration/migration_",run_no+1,".txt");
	}
	ofstream file_migration(f_name3);

	clock_t start, finish;
	double time_consume;

	time_consume = 0.0;
	start = clock();						// starts the clock
	
	srand((unsigned)time(0));
	m_rnd.randomize(seed);

	evaluations = 0;						// reset the NFFEs

	init_variables();
	init_pop();
	evaluate_pop(parent_pop, pop_size);
	find_best_index(parent_pop, pop_size);
	best_individual = parent_pop[best_index];

	gen = 1;								// current generation number
	int counter=0;							// only for restarting the population

	report_result(gen, file_process);
	report_diversity(gen, file_diversity);
	report_parameter(gen, file_migration);

	int feasible_flag = 0;					// check to get the feasible individual first
	flag_precision = 0;						// check to arrive the required value
	/* -------- add different optimizer here -------- */
	while ( (evaluations < max_NFFEs) )// && (gen < max_iterations) )
	{
		finish = clock();					// time consuming of this generation
		time_consume = (double)(finish-start)/CLOCKS_PER_SEC;

		// report the results in the screen
		/* comment the following routines to save the time_consuming */
		//display_result(gen);
		
		// ADD YOUR OPTIMIZER HERE
		switch(method) {
		case 1:// rank_JADE method
			run_rank_JADE_method();
			break;
		default:
			printf("The method selected does not exist.\n");
			exit(0);
		}

		gen++;

		find_best_index(parent_pop, pop_size);
		
		if (compare_ind(&parent_pop[best_index], &best_individual) != -1)
		{
			best_individual = parent_pop[best_index];
		}
		
		report_result(gen, file_process);
		report_diversity(gen, file_diversity);
		report_parameter(gen, file_migration);

		if (flag_precision == 0 && 
			fabs(MINIMIZE*best_individual.fitness - known_optimal) < PRECISION
			&& best_individual.feasible == 1)
		{
			flag_precision = 1;
			ofstream SummaryFile1;
			char     f_name00[150];
			if (batch_execution==0)	
			{
				SummaryFile1.open(".\\results\\evaluations.txt",ios::app);	
			}
			if (batch_execution==1) 
			{
				sprintf(f_name00, "%s%s%d%s%d%s%d%s", ".\\results", (func_index<10 ? ".\\f0" : ".\\f"), func_index, ".\\algorithm_", method, ".\\strategy_", strategy_ii, ".\\evaluations.txt");
				SummaryFile1.open(f_name00, ios::app);
			}			
			SummaryFile1<<setw(9)<<evaluations<<setw(8)<<time_consume<<endl;
			SummaryFile1.close();
			//break;		// һľͳӦֵ۴ʱʹ
		}
		if (N_of_constr != 0 && feasible_flag == 0 && best_individual.feasible == 1)
		{
			feasible_flag = 1;
			ofstream SummaryFile1;
			SummaryFile1.open(".\\results\\feasible_NFFEs.txt",ios::app);
			SummaryFile1<<setw(9)<<evaluations<<endl;
			SummaryFile1.close();
		}
	}
	/* -------- add different optimizer here -------- */
	
	printf("The total running time is %f s.\n", time_consume);
	if (index_of_normal != 0)
	{
		printf("The routine in optimizing f(%d) exits successfully.\n\n", index_of_normal);
	}

	SummaryFile.precision(15);
	SummaryFile<<setw(5)<<gen<<setw(9)<<evaluations
		<<setw(25)<<MINIMIZE * best_individual.fitness
		<<setw(8)<<time_consume
		<<setw(25)<<best_individual.constr_violation<<endl;

	// ŸԱ
	/**/
	if (batch_execution==0)	
	{		
		ofstream file_variable(".\\results\\x_variables.txt", ios::app);
		for (i=0;i<N_of_x;i++)
		{
			file_variable.precision(10);
			file_variable<<setw(18)<<best_individual.xreal[i];
		}
		file_variable<<endl;
		file_variable.close();
	}
	if (batch_execution==1) 
	{
		ofstream file_variable;
		char     f_name00[150];

		sprintf(f_name00, "%s%s%d%s%d%s%d%s", ".\\results", (func_index<10 ? ".\\f0" : ".\\f"), func_index, ".\\algorithm_", method, ".\\strategy_", strategy_ii, ".\\x_variables.txt");
		file_variable.open(f_name00, ios::app);

		for (i=0;i<N_of_x;i++)
		{
			file_variable.precision(10);
			file_variable<<setw(18)<<best_individual.xreal[i];
		}
		file_variable<<endl;
		file_variable.close();
	}

	// Ⱥ
	/*ofstream final_pop(".\\results\\pop.txt");
	final_pop.precision(8);
	for (int i=0;i<pop_size;i++)
	{
		for (int j=0;j<N_of_x;j++)
		{
			final_pop<<setw(15)<<parent_pop[i].xreal[j];
		}
		final_pop<<setw(15)<<parent_pop[i].fitness<<endl;
	}
	final_pop.close();*/
		
	file_process.close();
	file_diversity.close();
	file_migration.close();
	SummaryFile.close();
	
	return;
}

void CDE_method::shrink_bounary(CIndividual *pop, int size)
{
	double min_j[N_of_x];
	double max_j[N_of_x];
	int    i,j;

	// initialize the arrays
	for (j=0;j<N_of_x;j++)
	{
		min_j[j] = INF;
		max_j[j] = -INF;
	}

	// find the max and the min value of the variable in the current population
	for (i=0;i<size;i++)
	{
		for (j=0;j<N_of_x;j++)
		{
			if (pop[i].xreal[j] < min_j[j])
			{
				min_j[j] = pop[i].xreal[j];
			}
			if (pop[i].xreal[j] > max_j[j])
			{
				max_j[j] = pop[i].xreal[j];
			}
		}
	}

	// update the boundary of the search space
	for (j=0;j<N_of_x;j++)
	{
		lower_bounds[j] = min_j[j];
		upper_bounds[j] = max_j[j];
	}
}


/************************************************************************/
/* DE with rank-based mutation operators                                */
/************************************************************************/
void CDE_method::calculate_weights(tFitness *S_reward, int S_size, double *weights, int flag)
{
	int i;

	/* sort the rewards in descending order */
	int weight_index[1000];
	shell_sort_array_t(S_reward, weight_index, S_size);

	/* calculate the weights (selection rankability) with linear model */
	if (1==flag)
	{// rank-based weights: linear model
		for (i=0;i<S_size;i++)
		{
			weights[weight_index[i]] = (double)(i+0)/S_size;					// linear migration model
		}
	}
	else if (2==flag)
	{// rank-based weights: quadratic model
		for (i=0;i<S_size;i++)
		{
			weights[weight_index[i]] = pow( (double)(i+0)/S_size, 2.0 );		// quadratic migration model
		}
	}
	else if (3==flag)
	{// rank-based weights: cosine model
		for (i=0;i<S_size;i++)
		{
			weights[weight_index[i]] = 0.5*(-cos((i+0)*PI/S_size) + 1.0);		// sinusoidal migration model
		}
	}
	else
	{
		printf("The weights calculating is wrong.\n");
		exit(0);
	}
}

void CDE_method::rank_vector_selection(int base_index, int *index, int size, const double *rankability)
{
	assert(size>=2 && size<=5);

	int i = base_index;

	if (2==size)
	{// select two different vectors
		do {
			index[0] = m_rnd.rndint(0, pop_size-1);
		} while (index[0]==i || !m_rnd.flip(rankability[index[0]]));
		do {
			index[1] = m_rnd.rndint(0, pop_size-1);
		} while(index[1]==i || index[1]==index[0]);
	}
	else if (3==size)
	{// select three different vectors
		do {
			index[0] = m_rnd.rndint(0, pop_size-1);
		} while (index[0]==i || !m_rnd.flip(rankability[index[0]]));
		do {
			index[1] = m_rnd.rndint(0, pop_size-1);
		} while(index[1]==i || index[1]==index[0] || !m_rnd.flip(rankability[index[1]]));		
		do {
			index[2] = m_rnd.rndint(0, pop_size-1);
		} while(index[2]==i || index[2]==index[1] || index[2]==index[0] || m_rnd.flip(rankability[index[2]]));
	}
	else if (4==size)
	{// select five different vectors
		do {
			index[0] = m_rnd.rndint(0, pop_size-1);
		} while (index[0]==i || !m_rnd.flip(rankability[index[0]]));
		do {
			index[1] = m_rnd.rndint(0, pop_size-1);
		} while(index[1]==i || index[1]==index[0] || !m_rnd.flip(rankability[index[1]]));		
		do {
			index[2] = m_rnd.rndint(0, pop_size-1);
		} while(index[2]==i || index[2]==index[1] || index[2]==index[0]);			
		do {
			index[3] = m_rnd.rndint(0, pop_size-1);
		} while(index[3]==i || index[3]==index[2] || index[3]==index[1] || index[3]==index[0] || !m_rnd.flip(rankability[index[3]]));
	}
	
	else// if (5==size)
	{// select five different vectors
		do {
			index[0] = m_rnd.rndint(0, pop_size-1);
		} while (index[0]==i || !m_rnd.flip(rankability[index[0]]));
		do {
			index[1] = m_rnd.rndint(0, pop_size-1);
		} while(index[1]==i || index[1]==index[0] || !m_rnd.flip(rankability[index[1]]));		
		do {
			index[2] = m_rnd.rndint(0, pop_size-1);
		} while(index[2]==i || index[2]==index[1] || index[2]==index[0]);			
		do {
			index[3] = m_rnd.rndint(0, pop_size-1);
		} while(index[3]==i || index[3]==index[2] || index[3]==index[1] || index[3]==index[0] || !m_rnd.flip(rankability[index[3]]));
		do {
			index[4] = m_rnd.rndint(0, pop_size-1);
		} while(index[4]==i || index[4]==index[3] || index[4]==index[2] || index[4]==index[1] || index[4]==index[0]);
	}
}


void CDE_method::run_rank_JADE_method()
{
	int    i, j;
	int    r1, r2, r3;
	double low, up;

	double FF[population_size];
	double CR[population_size];

	/* Step 1: Initialize the JADE-related parameters */
	int    archive_index;	// if r3 is chosen from archive archive_index=1, else =0
	int    p_index = -1;
	if (gen == 1)
	{
		JADE_mu_CR   = 0.5;
		JADE_mu_FF   = 0.5;
		JADE_c       = 0.1;

		JADE_p       = 0.05;
		archive_size = 0;
	}

	/************************************************************************/
	/* Calculate the probabilities of each individual based on its fitness  */
	/************************************************************************/
	double   probability[population_size];
	tFitness fitness[population_size];
	for (i=0; i<pop_size; i++)
	{
		fitness[i] = parent_pop[i].fitness;
	}
	calculate_weights(fitness, pop_size, probability, weights_flag);
	int indices[10] = {0};
	/************************************************************************/

	sort_pop(parent_pop, pop_size);

	/* Generate the CR and F values */
	for (i=0;i<pop_size;i++)
	{
		// generate F value for each individual (individual level)
		do {
			FF[i] = m_rnd.cauchy(JADE_mu_FF, 0.1);
		} while (FF[i] <= 0.0);
		if (FF[i] > 1.0)	FF[i] = 1.0;

		// generate CR value for each dimension of each individual (component level)
		CR[i] = m_rnd.gaussian(JADE_mu_CR, 0.1);
		if (CR[i] < 0.0)	CR[i] = 0.0;
		if (CR[i] > 1.0)	CR[i] = 1.0;
	}

	/************************************************************************/
	int		binary_string[N_of_x], ttt;
	double	temp_cr;
	/************************************************************************/

	for (i=0;i<pop_size;i++)
	{
		/************************************************************************/
		/* crossover rate repairing technique                                   */
		/************************************************************************/
		int j_rand = m_rnd.rndint(0, N_of_x-1);
		ttt = 0;
		for (j=0;j<N_of_x;j++)
		{
			if (m_rnd.rndreal(0,1)<CR[i] || j==j_rand)
			{
				binary_string[j] = 1;
			}
			else
			{
				binary_string[j] = 0;
			}
			ttt += binary_string[j];
		}
		temp_cr = (double)ttt/(double)N_of_x;
		CR[i]   = temp_cr;		// comment this for no crossover rate repairing
		/************************************************************************/

		// randomly choose the p_best individual
		p_index = m_rnd.rndint(0, int(JADE_p*pop_size)-1);
		p_index = sort_index[p_index];

		// select three parents randomly
		rank_vector_selection(i, indices, 3, probability);
		r1 = indices[0];	r2 = indices[1];	r3 = indices[2];	

		/************************************************************************/
		/* For JADE strategies with archive                                     */
		/************************************************************************/
		if (m_strategy==3 || m_strategy==4)
		{
			// choose r3 from the union of parent population and the archive
			archive_index = 0;
			int k = pop_size+archive_size;
			r3 = m_rnd.rndint(0, k-1);
			if (r3>=0 && r3<pop_size)
			{// r3 is chosen from the parent population
				while(r3 == r2 || r3 == r1 || r3 == i)
				{
					r3 = m_rnd.rndint(0, pop_size-1);
				}
			}
			else
			{// r3 is chosen from the archive
				archive_index = 1;
				r3 = r3-pop_size;
			}
		}
		/************************************************************************/

		m_F = FF[i];

		//int j_rand = m_rnd.rndint(0, N_of_x-1);
		for (j=0;j<N_of_x;j++)
		{
			low = lower_bounds[j];
			up  = upper_bounds[j];

			if (binary_string[j]==1)
			{
				switch(m_strategy) {
				case 2:// DE/current-to-pbest mutation
					child_pop[i].xreal[j] = parent_pop[i].xreal[j] +
						m_F*(parent_pop[p_index].xreal[j]-parent_pop[i].xreal[j]) +
						m_F*(parent_pop[r2].xreal[j]-parent_pop[r3].xreal[j]);
					break;
				case 1:// DE/rand-to-pbest mutation
					child_pop[i].xreal[j] = parent_pop[r1].xreal[j] +
						m_F*(parent_pop[p_index].xreal[j]-parent_pop[r1].xreal[j]) +
						m_F*(parent_pop[r2].xreal[j]-parent_pop[r3].xreal[j]);
					break;
				case 3:// DE/current-to-pbest mutation with archive
					if (archive_index==0)
					{
						child_pop[i].xreal[j] = parent_pop[i].xreal[j] +
							m_F*(parent_pop[p_index].xreal[j]-parent_pop[i].xreal[j]) +
							m_F*(parent_pop[r2].xreal[j]-parent_pop[r3].xreal[j]);
					}
					else
					{
						child_pop[i].xreal[j] = parent_pop[i].xreal[j] +
							m_F*(parent_pop[p_index].xreal[j]-parent_pop[i].xreal[j]) +
							m_F*(parent_pop[r2].xreal[j]-archive_pop[r3].xreal[j]);
					}
					break;
				case 4:// DE/rand-to-pbest mutation with archive
					if (archive_index==0)
					{
						child_pop[i].xreal[j] = parent_pop[r1].xreal[j] +
							m_F*(parent_pop[p_index].xreal[j]-parent_pop[r1].xreal[j]) +
							m_F*(parent_pop[r2].xreal[j]-parent_pop[r3].xreal[j]);
					}
					else
					{
						child_pop[i].xreal[j] = parent_pop[r1].xreal[j] +
							m_F*(parent_pop[p_index].xreal[j]-parent_pop[r1].xreal[j]) +
							m_F*(parent_pop[r2].xreal[j]-archive_pop[r3].xreal[j]);
					}
					break;
				default:
					printf("The scheme in DE method does not exist.\n");
					exit(0);
				}
				
				if (child_pop[i].xreal[j]<low || child_pop[i].xreal[j]>up)
				{
					child_pop[i].xreal[j] = m_rnd.rndreal(low, up);
				}		
				assert(child_pop[i].xreal[j]>=low && child_pop[i].xreal[j]<=up);
			}
			else
			{
				child_pop[i].xreal[j] = parent_pop[i].xreal[j];
			}
		}
	}

	evaluate_pop(child_pop, pop_size);

	double		SS_FF[population_size], SS_CR[population_size];
	tFitness	S_reward[population_size], rr;
	int			SS_size = 0;						// The successful size of S_CR and S_F
	for (i=0;i<pop_size;i++)
	{
		int flag = compare_ind(&child_pop[i], &parent_pop[i]);
		if (flag != -1)
		{
			/************************************************************************/
			/* Calculate the relative fitness improvement                           */
			/************************************************************************/
			// get the absolute fitness improvement
			rr = parent_pop[i].fitness - child_pop[i].fitness; // for minimization
			if(parent_pop[i].fitness>=DBL_MAX)
			{
				rr = 1e100;
			}

			// calculate the relative fitness improvement
			if (child_pop[i].fitness > 0.0)
			{
				rr = (best_individual.fitness*rr)/child_pop[i].fitness;	// !!!
			}
			/************************************************************************/

			/* Step 3: Update the accepting flag and store the successful CR and F values */
			SS_FF[SS_size]     = FF[i];
			SS_CR[SS_size]     = CR[i];
			S_reward[SS_size]  = rr;
			SS_size++;
			/************************************************************************/

			/************************************************************************/
			/* Save the inferior parent into the archive                            */
			/************************************************************************/
			if (m_strategy==3 || m_strategy==4)
			{
				if (archive_size < pop_size)
				{
					archive_pop[archive_size] = parent_pop[i];
					archive_size++;
				}
				else
				{
					archive_size = pop_size;
					
					int ii = m_rnd.rndint(0, archive_size-1);
					archive_pop[ii] = parent_pop[i];
				}
			}
			/************************************************************************/

			// update the parent_pop with the better offspring
			parent_pop[i] = child_pop[i];
		}
	}

	/* Step 4: Update mu_CR and mu_F based on the successful CRs and Fs */
	if (SS_size != 0) 
	{
	
		double mean_cr  = 0.0;
		for (i=0;i<SS_size;i++)
		{
			mean_cr += SS_CR[i];
		}
		mean_cr    = mean_cr/((double)SS_size);
		JADE_mu_CR = (1-JADE_c) * JADE_mu_CR + JADE_c * mean_cr;

		// update the mu_F value
		double mean_ff = 0.0;
		double t1      = 0.0;
		double t2      = 0.0;
		for (i=0;i<SS_size;i++)
		{
			t1 += SS_FF[i]*SS_FF[i];
			t2 += SS_FF[i];
		}					
		mean_ff    = t1/t2;											// Lehmer mean

		JADE_mu_FF = (1-JADE_c)*JADE_mu_FF + JADE_c*mean_ff;
	}
}




